@@ -20,7 +20,6 @@
 
 #include "task-item.h"
 #include "task-list.h"
-#include "applet.h"
 
 #include <math.h>
 #include <glib/gi18n.h>
@@ -42,13 +41,15 @@ struct _TaskItemPrivate {
     GdkPixbuf    *pixbuf;
     GdkRectangle area;
     GTimeVal     urgent_time;
-    guint        timer;
+    guint        blink_timer;
     gboolean     mouse_over;
-    WindowPickerApplet *windowPickerApplet;
+    GdkMonitor  *monitor;
+    WpApplet    *windowPickerApplet;
 };
 
 enum {
     TASK_ITEM_CLOSED_SIGNAL,
+    TASK_ITEM_MONITOR_CHANGED,
     LAST_SIGNAL
 };
 
@@ -79,6 +80,8 @@ static const GtkTargetEntry drag_types[] = {
 
 static const gint n_drag_types = G_N_ELEMENTS(drag_types);
 
+static void task_item_close (TaskItem *item);
+
 static void update_hints (TaskItem *item) {
     GtkWidget *parent, *widget;
     GtkAllocation allocation_parent, allocation_widget;
@@ -140,12 +143,12 @@ static gboolean on_task_item_button_released (
         if (WNCK_IS_WORKSPACE (workspace)
             && workspace != wnck_screen_get_active_workspace (screen))
         {
-                wnck_workspace_activate (workspace, event->time);
+            wnck_workspace_activate (workspace, gtk_get_current_event_time ());
         }
         if (wnck_window_is_active (window)) {
             wnck_window_minimize (window);
         } else {
-            wnck_window_activate (window, event->time);
+            wnck_window_activate_transient (window, gtk_get_current_event_time ());
         }
     }
     return TRUE;
@@ -164,7 +167,7 @@ static void task_item_set_visibility (TaskItem *item) {
     window = priv->window;
     screen = priv->screen;
     workspace = wnck_screen_get_active_workspace (screen);
-    gboolean show_all = window_picker_applet_get_show_all_windows (priv->windowPickerApplet);
+    gboolean show_all = wp_applet_get_show_all_windows (priv->windowPickerApplet);
     gboolean show_window = FALSE;
     if (!wnck_window_is_skip_tasklist (window)) {
         if(workspace != NULL) { //this can happen sometimes
@@ -244,7 +247,7 @@ static GdkPixbuf *task_item_sized_pixbuf_for_window (
 static gboolean task_item_draw (
     GtkWidget      *widget,
     cairo_t *cr,
-    WindowPickerApplet* windowPickerApplet)
+    WpApplet* windowPickerApplet)
 {
     g_return_val_if_fail (widget != NULL, FALSE);
     g_return_val_if_fail (TASK_IS_ITEM (widget), FALSE);
@@ -256,10 +259,10 @@ static gboolean task_item_draw (
     GdkPixbuf *pbuf;
     area = priv->area;
     pbuf = priv->pixbuf;
-    gint size = MIN (area.height, area.width);
+    gint size = MIN (area.height, area.width) - 8;
     gboolean active = wnck_window_is_active (priv->window);
     /* load the GSettings key for gray icons */
-    gboolean icons_greyscale = window_picker_applet_get_icons_greyscale (priv->windowPickerApplet);
+    gboolean icons_greyscale = wp_applet_get_icons_greyscale (priv->windowPickerApplet);
     gboolean attention = wnck_window_or_transient_needs_attention (priv->window);
     if (GDK_IS_PIXBUF (pbuf) &&
         gdk_pixbuf_get_width (pbuf) != size &&
@@ -288,11 +291,11 @@ static gboolean task_item_draw (
         glow_x = area.width / 2;
         glow_y = area.height / 2;
         glow_pattern = cairo_pattern_create_radial (
-            area.x + glow_x, glow_y, glow_x * 0.6,
-            area.x + glow_x, glow_y, glow_x * 1.5
+            area.x + glow_x, area.y + glow_y, glow_x * 0.3,
+            area.x + glow_x, area.y + glow_y, glow_x * 1.4
         );
         cairo_pattern_add_color_stop_rgba (glow_pattern, 0, 1, 1, 1, 1);
-        cairo_pattern_add_color_stop_rgba (glow_pattern, 0.4, 1, 1, 1, 0);
+        cairo_pattern_add_color_stop_rgba (glow_pattern, 0.6, 1, 1, 1, 0);
         cairo_set_source (cr, glow_pattern);
         cairo_paint (cr);
     }
@@ -427,7 +430,7 @@ static gboolean on_blink (TaskItem *item) {
     if (wnck_window_or_transient_needs_attention (item->priv->window)) {
         return TRUE;
     } else {
-        item->priv->timer = 0;
+        item->priv->blink_timer = 0;
         return FALSE;
     }
 }
@@ -441,8 +444,8 @@ static void on_window_state_changed (
     g_return_if_fail (WNCK_IS_WINDOW (window));
     g_return_if_fail (TASK_IS_ITEM (taskItem));
     TaskItemPrivate *priv = taskItem->priv;
-    if (new_state & WNCK_WINDOW_STATE_URGENT && !priv->timer) {
-        priv->timer = g_timeout_add (30, (GSourceFunc)on_blink, taskItem);
+    if (new_state & WNCK_WINDOW_STATE_URGENT && !priv->blink_timer) {
+        priv->blink_timer = g_timeout_add (30, (GSourceFunc)on_blink, taskItem);
         g_get_current_time (&priv->urgent_time);
     }
     task_item_set_visibility (taskItem);
@@ -465,6 +468,65 @@ static void on_window_icon_changed (WnckWindow *window, TaskItem *item) {
     gtk_widget_queue_draw (GTK_WIDGET (item));
 }
 
+static void
+on_window_type_changed (WnckWindow *window,
+                        TaskItem *item)
+{
+    WnckWindowType type;
+
+    if (item->priv->window != window)
+        return;
+
+    type = wnck_window_get_window_type (window);
+
+    if (type == WNCK_WINDOW_DESKTOP ||
+        type == WNCK_WINDOW_DOCK ||
+        type == WNCK_WINDOW_SPLASHSCREEN ||
+        type == WNCK_WINDOW_MENU)
+      {
+          task_item_close (item);
+      }
+}
+
+static GdkMonitor *
+get_window_monitor (WnckWindow *window)
+{
+    gint x;
+    gint y;
+    gint w;
+    gint h;
+    GdkMonitor *window_monitor;
+    GdkDisplay *gdk_display;
+
+    wnck_window_get_geometry (window, &x, &y, &w, &h);
+
+    gdk_display = gdk_display_get_default ();
+    window_monitor = gdk_display_get_monitor_at_point (gdk_display,
+                                                       x + w / 2,
+                                                       y + h / 2);
+
+    return window_monitor;
+}
+
+static void
+on_window_geometry_changed (WnckWindow *window,
+                            TaskItem   *item)
+{
+    GdkMonitor *old_monitor;
+    GdkMonitor *window_monitor;
+
+    window_monitor = get_window_monitor (window);
+
+    old_monitor = item->priv->monitor;
+    if (old_monitor != window_monitor)
+      {
+        item->priv->monitor = window_monitor;
+
+        g_signal_emit (item, task_item_signals[TASK_ITEM_MONITOR_CHANGED], 0,
+                       old_monitor);
+      }
+}
+
 static void on_screen_active_window_changed (
     WnckScreen    *screen,
     WnckWindow    *old_window,
@@ -510,23 +572,57 @@ static void on_screen_window_closed (
     TaskItemPrivate *priv = item->priv;
     g_return_if_fail (WNCK_IS_WINDOW (priv->window));
     if (priv->window == window) {
-        g_signal_handlers_disconnect_by_func (screen,
-            G_CALLBACK (on_screen_window_closed), item);
-        g_signal_handlers_disconnect_by_func (screen,
-            G_CALLBACK (on_screen_active_window_changed), item);
-        g_signal_handlers_disconnect_by_func (screen,
-            G_CALLBACK (on_screen_active_workspace_changed), item);
-        g_signal_handlers_disconnect_by_func (screen,
-            G_CALLBACK (on_screen_window_closed), item);
-        g_signal_handlers_disconnect_by_func (window,
-            G_CALLBACK (on_window_workspace_changed), item);
-        g_signal_handlers_disconnect_by_func (window,
-            G_CALLBACK (on_window_state_changed), item);
-        g_signal_emit (G_OBJECT (item),
-            task_item_signals[TASK_ITEM_CLOSED_SIGNAL], 0);
+        task_item_close (item);
     }
 }
 
+static void
+disconnect_screen (TaskItem *item)
+{
+    WnckScreen *screen;
+
+    screen = item->priv->screen;
+
+    g_signal_handlers_disconnect_by_func (screen, on_screen_window_closed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (screen,
+                                          on_screen_active_window_changed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (screen,
+                                          on_screen_active_workspace_changed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (screen,
+                                          on_screen_active_viewport_changed,
+                                          item);
+}
+
+static void
+disconnect_window (TaskItem *item)
+{
+    WnckWindow *window;
+
+    window = item->priv->window;
+
+    g_signal_handlers_disconnect_by_func (window, on_window_workspace_changed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (window, on_window_state_changed,
+                                          item);
+    g_signal_handlers_disconnect_by_func (window, on_window_icon_changed, item);
+    g_signal_handlers_disconnect_by_func (window, on_window_type_changed, item);
+    g_signal_handlers_disconnect_by_func (window, on_window_geometry_changed,
+                                          item);
+}
+
+static void
+task_item_close (TaskItem   *item)
+{
+    disconnect_window (item);
+    disconnect_screen (item);
+
+    g_signal_emit (G_OBJECT (item),
+                   task_item_signals[TASK_ITEM_CLOSED_SIGNAL], 0);
+}
+
 static gboolean activate_window (GtkWidget *widget) {
     gint active;
     TaskItemPrivate *priv;
@@ -666,14 +762,14 @@ static void on_drag_received_data (
         gint active;
         switch (target_type) {
             case TARGET_WIDGET_DRAGGED: {
-                GtkWidget *taskList = window_picker_applet_get_tasks(item->priv->windowPickerApplet);
+                GtkWidget *taskList = wp_applet_get_tasks(item->priv->windowPickerApplet);
                 gpointer *data = (gpointer *) gtk_selection_data_get_data(selection_data);
                 g_assert(GTK_IS_WIDGET(*data));
 
                 GtkWidget *taskItem = GTK_WIDGET(*data);
                 g_assert(TASK_IS_ITEM(taskItem));
                 if(taskItem == widget) break; //source and target are identical
-                gint target_position = grid_get_pos(window_picker_applet_get_tasks(item->priv->windowPickerApplet), widget);
+                gint target_position = grid_get_pos(wp_applet_get_tasks(item->priv->windowPickerApplet), widget);
                 g_object_ref(taskItem);
                 gtk_box_reorder_child(GTK_BOX(taskList), taskItem, target_position);
                 g_object_unref(taskItem);
@@ -719,11 +815,21 @@ static void task_item_setup_atk (TaskItem *item) {
     atk_object_set_role (atk, ATK_ROLE_PUSH_BUTTON);
 }
 
+static void
+task_item_dispose (GObject *object)
+{
+    TaskItem *task_item = TASK_ITEM (object);
+
+    disconnect_window (task_item);
+    disconnect_screen (task_item);
+
+    G_OBJECT_CLASS (task_item_parent_class)->dispose (object);
+}
+
 static void task_item_finalize (GObject *object) {
     TaskItemPrivate *priv = TASK_ITEM (object)->priv;
-    /* remove timer */
-    if (priv->timer) {
-        g_source_remove (priv->timer);
+    if (priv->blink_timer) {
+        g_source_remove (priv->blink_timer);
     }
 
     if (GDK_IS_PIXBUF (priv->pixbuf)) {
@@ -735,6 +841,7 @@ static void task_item_finalize (GObject *object) {
 static void task_item_class_init (TaskItemClass *klass) {
     GObjectClass *obj_class      = G_OBJECT_CLASS (klass);
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+    obj_class->dispose = task_item_dispose;
     obj_class->finalize = task_item_finalize;
     widget_class->get_preferred_width = task_item_get_preferred_width;
     widget_class->get_preferred_height = task_item_get_preferred_height;
@@ -746,14 +853,18 @@ static void task_item_class_init (TaskItemClass *klass) {
         G_STRUCT_OFFSET (TaskItemClass, itemclosed),
         NULL, NULL,
         g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+
+    task_item_signals [TASK_ITEM_MONITOR_CHANGED] =
+        g_signal_new ("monitor-changed", TASK_TYPE_ITEM, G_SIGNAL_RUN_LAST,
+                      0, NULL, NULL, NULL, G_TYPE_NONE, 1, GDK_TYPE_MONITOR);
 }
 
 static void task_item_init (TaskItem *item) {
     TaskItemPrivate *priv = item->priv = TASK_ITEM_GET_PRIVATE (item);
-    priv->timer = 0;
+    priv->blink_timer = 0;
 }
 
-GtkWidget *task_item_new (WindowPickerApplet* windowPickerApplet, WnckWindow *window) {
+GtkWidget *task_item_new (WpApplet* windowPickerApplet, WnckWindow *window) {
     g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
     TaskItem *taskItem;
     TaskItemPrivate *priv;
@@ -774,6 +885,7 @@ GtkWidget *task_item_new (WindowPickerApplet* windowPickerApplet, WnckWindow *wi
     screen = wnck_window_get_screen (window);
     priv->screen = screen;
     priv->windowPickerApplet = windowPickerApplet;
+    priv->monitor = get_window_monitor (window);
 
     /** Drag and Drop code
      * This item can be both the target and the source of a drag and drop
@@ -833,6 +945,10 @@ GtkWidget *task_item_new (WindowPickerApplet* windowPickerApplet, WnckWindow *wi
         G_CALLBACK (on_window_state_changed), item);
     g_signal_connect (window, "icon-changed",
         G_CALLBACK (on_window_icon_changed), item);
+    g_signal_connect (window, "type-changed",
+        G_CALLBACK (on_window_type_changed), item);
+    g_signal_connect (window, "geometry-changed",
+        G_CALLBACK (on_window_geometry_changed), item);
     g_signal_connect(item, "draw",
         G_CALLBACK(task_item_draw), windowPickerApplet);
     g_signal_connect (item, "button-release-event",
@@ -851,3 +967,9 @@ GtkWidget *task_item_new (WindowPickerApplet* windowPickerApplet, WnckWindow *wi
     task_item_setup_atk (taskItem);
     return item;
 }
+
+GdkMonitor *
+task_item_get_monitor (TaskItem *item)
+{
+    return item->priv->monitor;
+}
